@@ -35,13 +35,11 @@ import os
 import re
 import tarfile
 
-import bzrlib
 from bzrlib import (
     chk_map,
     errors,
     generate_ids,
     osutils,
-    symbol_versioning,
     )
 """)
 
@@ -49,7 +47,6 @@ from bzrlib.errors import (
     BzrCheckError,
     BzrError,
     )
-from bzrlib.symbol_versioning import deprecated_in, deprecated_method
 from bzrlib.trace import mutter
 from bzrlib.static_tuple import StaticTuple
 
@@ -131,7 +128,23 @@ class InventoryEntry(object):
     RENAMED = 'renamed'
     MODIFIED_AND_RENAMED = 'modified and renamed'
 
-    __slots__ = []
+    __slots__ = ['file_id', 'revision', 'parent_id', 'name']
+
+    # Attributes that all InventoryEntry instances are expected to have, but
+    # that don't vary for all kinds of entry.  (e.g. symlink_target is only
+    # relevant to InventoryLink, so there's no reason to make every
+    # InventoryFile instance allocate space to hold a value for it.)
+    # Attributes that only vary for files: executable, text_sha1, text_size,
+    # text_id
+    executable = False
+    text_sha1 = None
+    text_size = None
+    text_id = None
+    # Attributes that only vary for symlinks: symlink_target
+    symlink_target = None
+    # Attributes that only vary for tree-references: reference_revision
+    reference_revision = None
+
 
     def detect_changes(self, old_entry):
         """Return a (text_modified, meta_modified) from this to old_entry.
@@ -176,16 +189,6 @@ class InventoryEntry(object):
                     candidates[ie.revision] = ie
         return candidates
 
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def get_tar_item(self, root, dp, now, tree):
-        """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(osutils.pathjoin(root, dp).encode('utf8'))
-        # TODO: would be cool to actually set it to the timestamp of the
-        # revision it was last changed
-        item.mtime = now
-        fileobj = self._put_in_tar(item, tree)
-        return item, fileobj
-
     def has_text(self):
         """Return true if the object this entry represents has textual data.
 
@@ -197,7 +200,7 @@ class InventoryEntry(object):
         """
         return False
 
-    def __init__(self, file_id, name, parent_id, text_id=None):
+    def __init__(self, file_id, name, parent_id):
         """Create an InventoryEntry
 
         The filename must be a single component, relative to the
@@ -214,16 +217,10 @@ class InventoryEntry(object):
         """
         if '/' in name or '\\' in name:
             raise errors.InvalidEntryName(name=name)
-        self.executable = False
-        self.revision = None
-        self.text_sha1 = None
-        self.text_size = None
         self.file_id = file_id
+        self.revision = None
         self.name = name
-        self.text_id = text_id
         self.parent_id = parent_id
-        self.symlink_target = None
-        self.reference_revision = None
 
     def kind_character(self):
         """Return a short kind indicator useful for appending to names."""
@@ -231,29 +228,6 @@ class InventoryEntry(object):
 
     known_kinds = ('file', 'directory', 'symlink')
 
-    def _put_in_tar(self, item, tree):
-        """populate item for stashing in a tar, and return the content stream.
-
-        If no content is available, return None.
-        """
-        raise BzrError("don't know how to export {%s} of kind %r" %
-                       (self.file_id, self.kind))
-
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def put_on_disk(self, dest, dp, tree):
-        """Create a representation of self on disk in the prefix dest.
-
-        This is a template method - implement _put_on_disk in subclasses.
-        """
-        fullpath = osutils.pathjoin(dest, dp)
-        self._put_on_disk(fullpath, tree)
-        # mutter("  export {%s} kind %s to %s", self.file_id,
-        #         self.kind, fullpath)
-
-    def _put_on_disk(self, fullpath, tree):
-        """Put this entry onto disk at fullpath, from tree tree."""
-        raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
-
     def sorted_children(self):
         return sorted(self.children.items())
 
@@ -397,47 +371,15 @@ class InventoryEntry(object):
         pass
 
 
-class RootEntry(InventoryEntry):
-
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
-
-    def _check(self, checker, rev_id):
-        """See InventoryEntry._check"""
-
-    def __init__(self, file_id):
-        self.file_id = file_id
-        self.children = {}
-        self.kind = 'directory'
-        self.parent_id = None
-        self.name = u''
-        self.revision = None
-        symbol_versioning.warn('RootEntry is deprecated as of bzr 0.10.'
-                               '  Please use InventoryDirectory instead.',
-                               DeprecationWarning, stacklevel=2)
-
-    def __eq__(self, other):
-        if not isinstance(other, RootEntry):
-            return NotImplemented
-
-        return (self.file_id == other.file_id) \
-               and (self.children == other.children)
-
-
 class InventoryDirectory(InventoryEntry):
     """A directory in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['children']
+
+    kind = 'directory'
 
     def _check(self, checker, rev_id):
         """See InventoryEntry._check"""
-        if (self.text_sha1 is not None or self.text_size is not None or
-            self.text_id is not None):
-            checker._report_items.append('directory {%s} has text in revision {%s}'
-                                % (self.file_id, rev_id))
         # In non rich root repositories we do not expect a file graph for the
         # root.
         if self.name == '' and not checker.rich_roots:
@@ -459,32 +401,25 @@ class InventoryDirectory(InventoryEntry):
     def __init__(self, file_id, name, parent_id):
         super(InventoryDirectory, self).__init__(file_id, name, parent_id)
         self.children = {}
-        self.kind = 'directory'
 
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return '/'
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.DIRTYPE
-        fileobj = None
-        item.name += '/'
-        item.size = 0
-        item.mode = 0755
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        os.mkdir(fullpath)
-
 
 class InventoryFile(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['text_sha1', 'text_size', 'text_id', 'executable']
+
+    kind = 'file'
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, parent_id)
+        self.text_sha1 = None
+        self.text_size = None
+        self.text_id = None
+        self.executable = False
 
     def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
@@ -533,31 +468,10 @@ class InventoryFile(InventoryEntry):
         """See InventoryEntry.has_text."""
         return True
 
-    def __init__(self, file_id, name, parent_id):
-        super(InventoryFile, self).__init__(file_id, name, parent_id)
-        self.kind = 'file'
-
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return ''
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.REGTYPE
-        fileobj = tree.get_file(self.file_id)
-        item.size = self.text_size
-        if tree.is_executable(self.file_id):
-            item.mode = 0755
-        else:
-            item.mode = 0644
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        osutils.pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
-        if tree.is_executable(self.file_id):
-            os.chmod(fullpath, 0755)
-
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.text_sha1 = work_tree.get_file_sha1(self.file_id, path=path)
@@ -595,16 +509,16 @@ class InventoryFile(InventoryEntry):
 class InventoryLink(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['symlink_target']
+
+    kind = 'symlink'
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryLink, self).__init__(file_id, name, parent_id)
+        self.symlink_target = None
 
     def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
-        if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
-            checker._report_items.append(
-               'symlink {%s} has text in revision {%s}'
-                    % (self.file_id, tree_revision_id))
         if self.symlink_target is None:
             checker._report_items.append(
                 'symlink {%s} has no target in revision {%s}'
@@ -648,30 +562,10 @@ class InventoryLink(InventoryEntry):
         differ = DiffSymlink(old_tree, new_tree, output_to)
         return differ.diff_symlink(old_target, new_target)
 
-    def __init__(self, file_id, name, parent_id):
-        super(InventoryLink, self).__init__(file_id, name, parent_id)
-        self.kind = 'symlink'
-
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return ''
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.SYMTYPE
-        fileobj = None
-        item.size = 0
-        item.mode = 0755
-        item.linkname = self.symlink_target
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        try:
-            os.symlink(self.symlink_target, fullpath)
-        except OSError,e:
-            raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
-
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.symlink_target = work_tree.get_symlink_target(self.file_id)
@@ -689,6 +583,8 @@ class InventoryLink(InventoryEntry):
 
 class TreeReference(InventoryEntry):
 
+    __slots__ = ['reference_revision']
+
     kind = 'tree-reference'
 
     def __init__(self, file_id, name, parent_id, revision=None,
@@ -940,7 +836,8 @@ class CommonInventory(object):
                 if ie.kind == 'directory':
                     descend(ie, child_path)
 
-        descend(self.root, u'')
+        if self.root is not None:
+            descend(self.root, u'')
         return accum
 
     def directories(self):
@@ -1282,9 +1179,6 @@ class Inventory(CommonInventory):
     def add(self, entry):
         """Add entry to inventory.
 
-        To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this.
-
         :return: entry
         """
         if entry.file_id in self._byid:
@@ -2241,17 +2135,13 @@ class CHKInventory(CommonInventory):
 class CHKInventoryDirectory(InventoryDirectory):
     """A directory in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', '_children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision',
-                 '_chk_inventory']
+    __slots__ = ['_children', '_chk_inventory']
 
     def __init__(self, file_id, name, parent_id, chk_inventory):
         # Don't call InventoryDirectory.__init__ - it isn't right for this
         # class.
         InventoryEntry.__init__(self, file_id, name, parent_id)
         self._children = None
-        self.kind = 'directory'
         self._chk_inventory = chk_inventory
 
     @property
