@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2007, 2008 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,14 +60,11 @@ from bzrlib.symbol_versioning import (
 
 DEFAULT_PLUGIN_PATH = None
 _loaded = False
+_plugins_disabled = False
 
-@deprecated_function(deprecated_in((2, 0, 0)))
-def get_default_plugin_path():
-    """Get the DEFAULT_PLUGIN_PATH"""
-    global DEFAULT_PLUGIN_PATH
-    if DEFAULT_PLUGIN_PATH is None:
-        DEFAULT_PLUGIN_PATH = osutils.pathjoin(config.config_dir(), 'plugins')
-    return DEFAULT_PLUGIN_PATH
+
+def are_plugins_disabled():
+    return _plugins_disabled
 
 
 def disable_plugins():
@@ -75,6 +72,8 @@ def disable_plugins():
 
     Future calls to load_plugins() will be ignored.
     """
+    global _plugins_disabled
+    _plugins_disabled = True
     load_plugins([])
 
 
@@ -82,6 +81,33 @@ def _strip_trailing_sep(path):
     return path.rstrip("\\/")
 
 
+def _get_specific_plugin_paths(paths):
+    """Returns the plugin paths from a string describing the associations.
+
+    :param paths: A string describing the paths associated with the plugins.
+
+    :returns: A list of (plugin name, path) tuples.
+
+    For example, if paths is my_plugin@/test/my-test:her_plugin@/production/her,
+    [('my_plugin', '/test/my-test'), ('her_plugin', '/production/her')] 
+    will be returned.
+
+    Note that ':' in the example above depends on the os.
+    """
+    if not paths:
+        return []
+    specs = []
+    for spec in paths.split(os.pathsep):
+        try:
+            name, path = spec.split('@')
+        except ValueError:
+            raise errors.BzrCommandError(
+                '"%s" is not a valid <plugin_name>@<plugin_path> description '
+                % spec)
+        specs.append((name, path))
+    return specs
+
+
 def set_plugins_path(path=None):
     """Set the path for plugins to be loaded from.
 
@@ -92,6 +118,17 @@ def set_plugins_path(path=None):
     if path is None:
         path = get_standard_plugins_path()
     _mod_plugins.__path__ = path
+    PluginImporter.reset()
+    # Set up a blacklist for disabled plugins
+    disabled_plugins = os.environ.get('BZR_DISABLE_PLUGINS', None)
+    if disabled_plugins is not None:
+        for name in disabled_plugins.split(os.pathsep):
+            PluginImporter.blacklist.add('bzrlib.plugins.' + name)
+    # Set up a the specific paths for plugins
+    for plugin_name, plugin_path in _get_specific_plugin_paths(os.environ.get(
+            'BZR_PLUGINS_AT', None)):
+            PluginImporter.specific_paths[
+                'bzrlib.plugins.%s' % plugin_name] = plugin_path
     return path
 
 
@@ -180,12 +217,12 @@ def get_standard_plugins_path():
     paths = []
     for p in env_paths + defaults:
         if p.startswith('+'):
-            # Resolve reference if they are known
+            # Resolve references if they are known
             try:
                 p = refs[p[1:]]
             except KeyError:
-                # Leave them untouched otherwise, user may have paths starting
-                # with '+'...
+                # Leave them untouched so user can still use paths starting
+                # with '+'
                 pass
         _append_new_path(paths, p)
 
@@ -203,7 +240,7 @@ def load_plugins(path=None):
     files (and whatever other extensions are used in the platform,
     such as *.pyd).
 
-    load_from_dirs() provides the underlying mechanism and is called with
+    load_from_path() provides the underlying mechanism and is called with
     the default directory list to provide the normal behaviour.
 
     :param path: The list of paths to search for plugins.  By default,
@@ -232,6 +269,11 @@ def load_from_path(dirs):
 
     The python module path for bzrlib.plugins will be modified to be 'dirs'.
     """
+    # Explicitly load the plugins with a specific path
+    for fullname, path in PluginImporter.specific_paths.iteritems():
+        name = fullname[len('bzrlib.plugins.'):]
+        _load_plugin_module(name, path)
+
     # We need to strip the trailing separators here as well as in the
     # set_plugins_path function because calling code can pass anything in to
     # this function, and since it sets plugins.__path__, it should set it to
@@ -251,70 +293,99 @@ def load_from_path(dirs):
 load_from_dirs = load_from_path
 
 
+def _find_plugin_module(dir, name):
+    """Check if there is a valid python module that can be loaded as a plugin.
+
+    :param dir: The directory where the search is performed.
+    :param path: An existing file path, either a python file or a package
+        directory.
+
+    :return: (name, path, description) name is the module name, path is the
+        file to load and description is the tuple returned by
+        imp.get_suffixes().
+    """
+    path = osutils.pathjoin(dir, name)
+    if os.path.isdir(path):
+        # Check for a valid __init__.py file, valid suffixes depends on -O and
+        # can be .py, .pyc and .pyo
+        for suffix, mode, kind in imp.get_suffixes():
+            if kind not in (imp.PY_SOURCE, imp.PY_COMPILED):
+                # We don't recognize compiled modules (.so, .dll, etc)
+                continue
+            init_path = osutils.pathjoin(path, '__init__' + suffix)
+            if os.path.isfile(init_path):
+                return name, init_path, (suffix, mode, kind)
+    else:
+        for suffix, mode, kind in imp.get_suffixes():
+            if name.endswith(suffix):
+                # Clean up the module name
+                name = name[:-len(suffix)]
+                if kind == imp.C_EXTENSION and name.endswith('module'):
+                    name = name[:-len('module')]
+                return name, path, (suffix, mode, kind)
+    # There is no python module here
+    return None, None, (None, None, None)
+
+
+def _load_plugin_module(name, dir):
+    """Load plugin name from dir.
+
+    :param name: The plugin name in the bzrlib.plugins namespace.
+    :param dir: The directory the plugin is loaded from for error messages.
+    """
+    if ('bzrlib.plugins.%s' % name) in PluginImporter.blacklist:
+        return
+    try:
+        exec "import bzrlib.plugins.%s" % name in {}
+    except KeyboardInterrupt:
+        raise
+    except errors.IncompatibleAPI, e:
+        trace.warning("Unable to load plugin %r. It requested API version "
+            "%s of module %s but the minimum exported version is %s, and "
+            "the maximum is %s" %
+            (name, e.wanted, e.api, e.minimum, e.current))
+    except Exception, e:
+        trace.warning("%s" % e)
+        if re.search('\.|-| ', name):
+            sanitised_name = re.sub('[-. ]', '_', name)
+            if sanitised_name.startswith('bzr_'):
+                sanitised_name = sanitised_name[len('bzr_'):]
+            trace.warning("Unable to load %r in %r as a plugin because the "
+                    "file path isn't a valid module name; try renaming "
+                    "it to %r." % (name, dir, sanitised_name))
+        else:
+            trace.warning('Unable to load plugin %r from %r' % (name, dir))
+        trace.log_exception_quietly()
+        if 'error' in debug.debug_flags:
+            trace.print_exception(sys.exc_info(), sys.stderr)
+
+
 def load_from_dir(d):
     """Load the plugins in directory d.
 
     d must be in the plugins module path already.
+    This function is called once for each directory in the module path.
     """
-    # Get the list of valid python suffixes for __init__.py?
-    # this includes .py, .pyc, and .pyo (depending on if we are running -O)
-    # but it doesn't include compiled modules (.so, .dll, etc)
-    valid_suffixes = [suffix for suffix, mod_type, flags in imp.get_suffixes()
-                              if flags in (imp.PY_SOURCE, imp.PY_COMPILED)]
-    package_entries = ['__init__'+suffix for suffix in valid_suffixes]
     plugin_names = set()
-    for f in os.listdir(d):
-        path = osutils.pathjoin(d, f)
-        if os.path.isdir(path):
-            for entry in package_entries:
-                # This directory should be a package, and thus added to
-                # the list
-                if os.path.isfile(osutils.pathjoin(path, entry)):
-                    break
-            else: # This directory is not a package
-                continue
-        else:
-            for suffix_info in imp.get_suffixes():
-                if f.endswith(suffix_info[0]):
-                    f = f[:-len(suffix_info[0])]
-                    if suffix_info[2] == imp.C_EXTENSION and f.endswith('module'):
-                        f = f[:-len('module')]
-                    break
+    for p in os.listdir(d):
+        name, path, desc = _find_plugin_module(d, p)
+        if name is not None:
+            if name == '__init__':
+                # We do nothing with the __init__.py file in directories from
+                # the bzrlib.plugins module path, we may want to, one day
+                # -- vila 20100316.
+                continue # We don't load __init__.py in the plugins dirs
+            elif getattr(_mod_plugins, name, None) is not None:
+                # The module has already been loaded from another directory
+                # during a previous call.
+                # FIXME: There should be a better way to report masked plugins
+                # -- vila 20100316
+                trace.mutter('Plugin name %s already loaded', name)
             else:
-                continue
-        if f == '__init__':
-            continue # We don't load __init__.py again in the plugin dir
-        elif getattr(_mod_plugins, f, None):
-            trace.mutter('Plugin name %s already loaded', f)
-        else:
-            # trace.mutter('add plugin name %s', f)
-            plugin_names.add(f)
+                plugin_names.add(name)
 
     for name in plugin_names:
-        try:
-            exec "import bzrlib.plugins.%s" % name in {}
-        except KeyboardInterrupt:
-            raise
-        except errors.IncompatibleAPI, e:
-            trace.warning("Unable to load plugin %r. It requested API version "
-                "%s of module %s but the minimum exported version is %s, and "
-                "the maximum is %s" %
-                (name, e.wanted, e.api, e.minimum, e.current))
-        except Exception, e:
-            trace.warning("%s" % e)
-            ## import pdb; pdb.set_trace()
-            if re.search('\.|-| ', name):
-                sanitised_name = re.sub('[-. ]', '_', name)
-                if sanitised_name.startswith('bzr_'):
-                    sanitised_name = sanitised_name[len('bzr_'):]
-                trace.warning("Unable to load %r in %r as a plugin because the "
-                        "file path isn't a valid module name; try renaming "
-                        "it to %r." % (name, d, sanitised_name))
-            else:
-                trace.warning('Unable to load plugin %r from %r' % (name, d))
-            trace.log_exception_quietly()
-            if 'error' in debug.debug_flags:
-                trace.print_exception(sys.exc_info(), sys.stderr)
+        _load_plugin_module(name, d)
 
 
 def plugins():
@@ -477,3 +548,81 @@ class PlugIn(object):
         return version_string
 
     __version__ = property(_get__version__)
+
+
+class _PluginImporter(object):
+    """An importer tailored to bzr specific needs.
+
+    This is a singleton that takes care of:
+    - disabled plugins specified in 'blacklist',
+    - plugins that needs to be loaded from specific directories.
+    """
+
+    def __init__(self):
+        self.reset()
+
+    def reset(self):
+        self.blacklist = set()
+        self.specific_paths = {}
+
+    def find_module(self, fullname, parent_path=None):
+        """Search a plugin module.
+
+        Disabled plugins raise an import error, plugins with specific paths
+        returns a specific loader.
+
+        :return: None if the plugin doesn't need special handling, self
+            otherwise.
+        """
+        if not fullname.startswith('bzrlib.plugins.'):
+            return None
+        if fullname in self.blacklist:
+            raise ImportError('%s is disabled' % fullname)
+        if fullname in self.specific_paths:
+            return self
+        return None
+
+    def load_module(self, fullname):
+        """Load a plugin from a specific directory."""
+        # We are called only for specific paths
+        plugin_path = self.specific_paths[fullname]
+        loading_path = None
+        if os.path.isdir(plugin_path):
+            for suffix, mode, kind in imp.get_suffixes():
+                if kind not in (imp.PY_SOURCE, imp.PY_COMPILED):
+                    # We don't recognize compiled modules (.so, .dll, etc)
+                    continue
+                init_path = osutils.pathjoin(plugin_path, '__init__' + suffix)
+                if os.path.isfile(init_path):
+                    # We've got a module here and load_module needs specific
+                    # parameters.
+                    loading_path = plugin_path
+                    suffix = ''
+                    mode = ''
+                    kind = imp.PKG_DIRECTORY
+                    break
+        else:
+            for suffix, mode, kind in imp.get_suffixes():
+                if plugin_path.endswith(suffix):
+                    loading_path = plugin_path
+                    break
+        if loading_path is None:
+            raise ImportError('%s cannot be loaded from %s'
+                              % (fullname, plugin_path))
+        if kind is imp.PKG_DIRECTORY:
+            f = None
+        else:
+            f = open(loading_path, mode)
+        try:
+            mod = imp.load_module(fullname, f, loading_path,
+                                  (suffix, mode, kind))
+            mod.__package__ = fullname
+            return mod
+        finally:
+            if f is not None:
+                f.close()
+
+
+# Install a dedicated importer for plugins requiring special handling
+PluginImporter = _PluginImporter()
+sys.meta_path.append(PluginImporter)
