@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,18 +15,16 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import base64
-import os
 from StringIO import StringIO
 import urlparse
 import xmlrpclib
 
 from bzrlib import (
     config,
-    osutils,
     tests,
     ui,
     )
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import TestCaseWithTransport
 
 # local import
 from bzrlib.plugins.launchpad.lp_registration import (
@@ -61,6 +59,25 @@ class InstrumentedXMLRPCConnection(object):
         """
         return (200, 'OK', [])
 
+    def getresponse(self, buffering=True):
+        """Fake the http reply.
+
+        This is used when running on Python 2.7, where xmlrpclib uses
+        httplib.HTTPConnection in a different way than before.
+        """
+        class FakeHttpResponse(object):
+
+            def __init__(self, status, reason, headers, body):
+                self.status = status
+                self.reason = reason
+                self.headers = headers
+                self.body = body
+
+            def read(self, size=-1):
+                return self.body.read(size)
+
+        return FakeHttpResponse(200, 'OK', [], self.getfile())
+
     def getfile(self):
         """Return a fake file containing the response content."""
         return StringIO('''\
@@ -85,6 +102,7 @@ class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
     def __init__(self, testcase, expect_auth):
         self.testcase = testcase
         self.expect_auth = expect_auth
+        self._connection = (None, None)
 
     def make_connection(self, host):
         host, http_headers, x509 = self.get_host_info(host)
@@ -133,14 +151,11 @@ class MockLaunchpadService(LaunchpadService):
 
 
 class TestBranchRegistration(TestCaseWithTransport):
-    SAMPLE_URL = 'http://bazaar-vcs.org/bzr/bzr.dev/'
-    SAMPLE_OWNER = 'jhacker@foo.com'
-    SAMPLE_BRANCH_ID = 'bzr.dev'
 
     def setUp(self):
         super(TestBranchRegistration, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_register_help(self):
         """register-branch accepts --help"""
@@ -299,7 +314,7 @@ class TestGatherUserCredentials(tests.TestCaseInTempDir):
     def setUp(self):
         super(TestGatherUserCredentials, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_gather_user_credentials_has_password(self):
         service = LaunchpadService()
