@@ -28,7 +28,6 @@ from bzrlib import (
     symbol_versioning,
     )
 from bzrlib.osutils import contains_whitespace
-from bzrlib.progress import DummyProgress
 
 NULL_REVISION="null:"
 CURRENT_REVISION="current:"
@@ -122,21 +121,6 @@ class Revision(object):
         else:
             return ''
 
-    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((1, 13, 0)))
-    def get_apparent_author(self):
-        """Return the apparent author of this revision.
-
-        This method is deprecated in favour of get_apparent_authors.
-
-        If the revision properties contain any author names,
-        return the first. Otherwise return the committer name.
-        """
-        authors = self.get_apparent_authors()
-        if authors:
-            return authors[0]
-        else:
-            return None
-
     def get_apparent_authors(self):
         """Return the apparent authors of this revision.
 
