@@ -18,22 +18,67 @@ import sys
 
 from bzrlib import (
     delta as _mod_delta,
+    hooks as _mod_hooks,
     log,
     osutils,
-    tree,
     tsort,
     revision as _mod_revision,
     )
 import bzrlib.errors as errors
-from bzrlib.osutils import is_inside_any
-from bzrlib.symbol_versioning import (deprecated_function,
-        )
 from bzrlib.trace import mutter, warning
 
 # TODO: when showing single-line logs, truncate to the width of the terminal
 # if known, but only if really going to the terminal (not into a file)
 
 
+def report_changes(to_file, old, new, specific_files, 
+                   show_short_reporter, show_long_callback, 
+                   short=False, want_unchanged=False, 
+                   want_unversioned=False, show_ids=False):
+    """Display summary of changes.
+
+    This compares two trees with regards to a list of files, and delegates 
+    the display to underlying elements.
+
+    For short output, it creates an iterator on all changes, and lets a given
+    reporter display these changes.
+
+    For stantard output, it creates a delta of the changes, and forwards it
+    to a callback
+
+    :param to_file: If set, write to this file (default stdout.)
+    :param old: Start tree for the comparison
+    :param end: End tree for the comparison
+    :param specific_files: If set, a list of filenames whose status should be
+        shown.  It is an error to give a filename that is not in the working
+        tree, or in the working inventory or in the basis inventory.
+    :param show_short_reporter: Reporter in charge of display for short output
+    :param show_long_callback: Callback in charge of display for normal output
+    :param short: If True, gives short SVN-style status lines.
+    :param want_unchanged: Deprecated parameter. If set, includes unchanged
+        files.
+    :param show_ids: If set, includes each file's id.
+    :param want_unversioned: If False, only shows versioned files.
+    """
+
+    if short:
+        changes = new.iter_changes(old, want_unchanged, specific_files,
+            require_versioned=False, want_unversioned=want_unversioned)
+        _mod_delta.report_changes(changes, show_short_reporter)
+        
+    else:
+        delta = new.changes_from(old, want_unchanged=want_unchanged,
+                              specific_files=specific_files,
+                              want_unversioned=want_unversioned)
+        # filter out unknown files. We may want a tree method for
+        # this
+        delta.unversioned = [unversioned for unversioned in
+            delta.unversioned if not new.is_ignored(unversioned[0])]
+        show_long_callback(to_file, delta, 
+                           show_ids=show_ids,
+                           show_unchanged=want_unchanged)
+
+
 def show_tree_status(wt, show_unchanged=None,
                      specific_files=None,
                      show_ids=False,
@@ -42,7 +87,8 @@ def show_tree_status(wt, show_unchanged=None,
                      revision=None,
                      short=False,
                      verbose=False,
-                     versioned=False):
+                     versioned=False,
+                     show_long_callback=_mod_delta.report_delta):
     """Display summary of changes.
 
     By default this compares the working tree to a previous revision.
@@ -71,6 +117,8 @@ def show_tree_status(wt, show_unchanged=None,
     :param verbose: If True, show all merged revisions, not just
         the merge tips
     :param versioned: If True, only shows versioned files.
+    :param show_long_callback: A callback: message = show_long_callback(to_file, delta, 
+        show_ids, show_unchanged, indent, filter), only used with the long output
     """
     if show_unchanged is not None:
         warn("show_tree_status with show_unchanged has been deprecated "
@@ -103,27 +151,36 @@ def show_tree_status(wt, show_unchanged=None,
         old.lock_read()
         new.lock_read()
         try:
+            for hook in hooks['pre_status']:
+                hook(StatusHookParams(old, new, to_file, versioned,
+                    show_ids, short, verbose))
+
             specific_files, nonexistents \
                 = _filter_nonexistent(specific_files, old, new)
             want_unversioned = not versioned
-            if short:
-                changes = new.iter_changes(old, show_unchanged, specific_files,
-                    require_versioned=False, want_unversioned=want_unversioned)
-                reporter = _mod_delta._ChangeReporter(output_file=to_file,
-                    unversioned_filter=new.is_ignored)
-                _mod_delta.report_changes(changes, reporter)
-            else:
-                delta = new.changes_from(old, want_unchanged=show_unchanged,
-                                      specific_files=specific_files,
-                                      want_unversioned=want_unversioned)
-                # filter out unknown files. We may want a tree method for
-                # this
-                delta.unversioned = [unversioned for unversioned in
-                    delta.unversioned if not new.is_ignored(unversioned[0])]
-                delta.show(to_file,
-                           show_ids=show_ids,
-                           show_unchanged=show_unchanged,
-                           short_status=False)
+
+            # Reporter used for short outputs
+            reporter = _mod_delta._ChangeReporter(output_file=to_file,
+                unversioned_filter=new.is_ignored)
+            report_changes(to_file, old, new, specific_files, 
+                           reporter, show_long_callback, 
+                           short=short, want_unchanged=show_unchanged, 
+                           want_unversioned=want_unversioned, show_ids=show_ids)
+
+            # show the ignored files among specific files (i.e. show the files
+            # identified from input that we choose to ignore). 
+            if specific_files is not None:
+                # Ignored files is sorted because specific_files is already sorted
+                ignored_files = [specific for specific in
+                    specific_files if new.is_ignored(specific)]
+                if len(ignored_files) > 0 and not short:
+                    to_file.write("ignored:\n")
+                    prefix = ' '
+                else:
+                    prefix = 'I  '
+                for ignored_file in ignored_files:
+                    to_file.write("%s %s\n" % (prefix, ignored_file))
+
             # show the new conflicts only for now. XXX: get them from the
             # delta.
             conflicts = new.conflicts()
@@ -158,6 +215,9 @@ def show_tree_status(wt, show_unchanged=None,
                 show_pending_merges(new, to_file, short, verbose=verbose)
             if nonexistents:
                 raise errors.PathsDoNotExist(nonexistents)
+            for hook in hooks['post_status']:
+                hook(StatusHookParams(old, new, to_file, versioned,
+                    show_ids, short, verbose))
         finally:
             old.unlock()
             new.unlock()
@@ -304,3 +364,105 @@ def _filter_nonexistent(orig_paths, old_tree, new_tree):
     # their groups individually.  But for consistency of this
     # function's API, it's better to sort both than just 'nonexistent'.
     return sorted(remaining), sorted(nonexistent)
+
+
+class StatusHooks(_mod_hooks.Hooks):
+    """A dictionary mapping hook name to a list of callables for status hooks.
+
+    e.g. ['post_status'] Is the list of items to be called when the
+    status command has finished printing the status.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        _mod_hooks.Hooks.__init__(self)
+        self.create_hook(_mod_hooks.HookPoint('post_status',
+            "Called with argument StatusHookParams after Bazaar has "
+            "displayed the status. StatusHookParams has the attributes "
+            "(old_tree, new_tree, to_file, versioned, show_ids, short, "
+            "verbose). The last four arguments correspond to the command "
+            "line options specified by the user for the status command. "
+            "to_file is the output stream for writing.",
+            (2, 3), None))
+        self.create_hook(_mod_hooks.HookPoint('pre_status',
+            "Called with argument StatusHookParams before Bazaar "
+            "displays the status. StatusHookParams has the attributes "
+            "(old_tree, new_tree, to_file, versioned, show_ids, short, "
+            "verbose). The last four arguments correspond to the command "
+            "line options specified by the user for the status command. "
+            "to_file is the output stream for writing.",
+            (2, 3), None))
+
+
+class StatusHookParams(object):
+    """Object holding parameters passed to post_status hooks.
+
+    :ivar old_tree: Start tree (basis tree) for comparison.
+    :ivar new_tree: Working tree.
+    :ivar to_file: If set, write to this file.
+    :ivar versioned: Show only versioned files.
+    :ivar show_ids: Show internal object ids.
+    :ivar short: Use short status indicators.
+    :ivar verbose: Verbose flag.
+    """
+
+    def __init__(self, old_tree, new_tree, to_file, versioned, show_ids,
+            short, verbose):
+        """Create a group of post_status hook parameters.
+
+        :param old_tree: Start tree (basis tree) for comparison.
+        :param new_tree: Working tree.
+        :param to_file: If set, write to this file.
+        :param versioned: Show only versioned files.
+        :param show_ids: Show internal object ids.
+        :param short: Use short status indicators.
+        :param verbose: Verbose flag.
+        """
+        self.old_tree = old_tree
+        self.new_tree = new_tree
+        self.to_file = to_file
+        self.versioned = versioned
+        self.show_ids = show_ids
+        self.short = short
+        self.verbose = verbose
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    def __repr__(self):
+        return "<%s(%s, %s, %s, %s, %s, %s, %s)>" % (self.__class__.__name__,
+            self.old_tree, self.new_tree, self.to_file, self.versioned,
+            self.show_ids, self.short, self.verbose)
+
+
+def _show_shelve_summary(params):
+    """post_status hook to display a summary of shelves.
+
+    :param params: StatusHookParams.
+    """
+    get_shelf_manager = getattr(params.new_tree, 'get_shelf_manager', None)
+    if get_shelf_manager is None:
+        return
+    manager = get_shelf_manager()
+    shelves = manager.active_shelves()
+    if shelves:
+        singular = '%d shelf exists. '
+        plural = '%d shelves exist. '
+        if len(shelves) == 1:
+            fmt = singular
+        else:
+            fmt = plural
+        params.to_file.write(fmt % len(shelves))
+        params.to_file.write('See "bzr shelve --list" for details.\n')
+
+
+hooks = StatusHooks()
+
+
+hooks.install_named_hook('post_status', _show_shelve_summary,
+    'bzr status')
+
