@@ -23,9 +23,9 @@ import os
 
 from bzrlib import (
     branch,
+    branchbuilder,
     bzrdir,
     conflicts,
-    errors,
     merge_directive,
     osutils,
     tests,
