@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,26 +15,21 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from cStringIO import StringIO
-import errno
 import re
-import socket
-import threading
-import time
 import urllib2
-import urlparse
 
 
 from bzrlib import (
     errors,
     osutils,
     tests,
+    transport,
+    )
+from bzrlib.smart import (
+    medium,
     )
-from bzrlib.smart import medium, protocol
 from bzrlib.tests import http_server
-from bzrlib.transport import (
-    chroot,
-    get_transport,
-    )
+from bzrlib.transport import chroot
 
 
 class HTTPServerWithSmarts(http_server.HttpServer):
@@ -56,11 +51,12 @@ class SmartRequestHandler(http_server.TestingHTTPRequestHandler):
 
     def do_POST(self):
         """Hand the request off to a smart server instance."""
-        backing = get_transport(self.server.test_case_server._home_dir)
+        backing = transport.get_transport(
+            self.server.test_case_server._home_dir)
         chroot_server = chroot.ChrootServer(backing)
         chroot_server.start_server()
         try:
-            t = get_transport(chroot_server.get_url())
+            t = transport.get_transport(chroot_server.get_url())
             self.do_POST_inner(t)
         finally:
             chroot_server.stop_server()
@@ -107,18 +103,21 @@ class TestCaseWithWebserver(tests.TestCaseWithTransport):
     backed by regular disk files.
     """
 
-    # This can be overriden or parametrized by daughter clasess if needed, but
-    # it must exist so that the create_transport_readonly_server() method can
-    # propagate it.
+    # These attributes can be overriden or parametrized by daughter clasess if
+    # needed, but must exist so that the create_transport_readonly_server()
+    # method (or any method creating an http(s) server) can propagate it.
     _protocol_version = None
+    _url_protocol = 'http'
 
     def setUp(self):
         super(TestCaseWithWebserver, self).setUp()
         self.transport_readonly_server = http_server.HttpServer
 
     def create_transport_readonly_server(self):
-        return self.transport_readonly_server(
+        server = self.transport_readonly_server(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
 
 class TestCaseWithTwoWebservers(TestCaseWithWebserver):
@@ -137,8 +136,10 @@ class TestCaseWithTwoWebservers(TestCaseWithWebserver):
 
         This is mostly a hook for daughter classes.
         """
-        return self.transport_secondary_server(
+        server = self.transport_secondary_server(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def get_secondary_server(self):
         """Get the server instance for the secondary transport."""
@@ -147,6 +148,15 @@ class TestCaseWithTwoWebservers(TestCaseWithWebserver):
             self.start_server(self.__secondary_server)
         return self.__secondary_server
 
+    def get_secondary_url(self, relpath=None):
+        base = self.get_secondary_server().get_url()
+        return self._adjust_url(base, relpath)
+
+    def get_secondary_transport(self, relpath=None):
+        t = transport.get_transport(self.get_secondary_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
 
 class ProxyServer(http_server.HttpServer):
     """A proxy test server for http transports."""
@@ -226,20 +236,39 @@ class TestCaseWithRedirectedWebserver(TestCaseWithTwoWebservers):
    The 'old' server is redirected to the 'new' server.
    """
 
+   def setUp(self):
+       super(TestCaseWithRedirectedWebserver, self).setUp()
+       # The redirections will point to the new server
+       self.new_server = self.get_readonly_server()
+       # The requests to the old server will be redirected to the new server
+       self.old_server = self.get_secondary_server()
+
    def create_transport_secondary_server(self):
        """Create the secondary server redirecting to the primary server"""
        new = self.get_readonly_server()
        redirecting = HTTPServerRedirecting(
            protocol_version=self._protocol_version)
        redirecting.redirect_to(new.host, new.port)
+       redirecting._url_protocol = self._url_protocol
        return redirecting
 
-   def setUp(self):
-       super(TestCaseWithRedirectedWebserver, self).setUp()
-       # The redirections will point to the new server
-       self.new_server = self.get_readonly_server()
-       # The requests to the old server will be redirected
-       self.old_server = self.get_secondary_server()
+   def get_old_url(self, relpath=None):
+        base = self.old_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_old_transport(self, relpath=None):
+        t = transport.get_transport(self.get_old_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
+   def get_new_url(self, relpath=None):
+        base = self.new_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_new_transport(self, relpath=None):
+        t = transport.get_transport(self.get_new_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
 
 
 class AuthRequestHandler(http_server.TestingHTTPRequestHandler):
