@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@ from bzrlib import (
 from bzrlib.inventory import ROOT_ID
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_repository import TestCaseWithRepository
-from bzrlib.transport import get_transport
 
 
 class TestFetchSameRepository(TestCaseWithRepository):
@@ -355,7 +354,7 @@ class TestSource(TestCaseWithRepository):
     """Tests for/about the results of Repository._get_source."""
 
     def test_no_absent_records_in_stream_with_ghosts(self):
-        # XXX: Arguably should be in interrepository_implementations but
+        # XXX: Arguably should be in per_interrepository but
         # doesn't actually gain coverage there; need a specific set of
         # permutations to cover it.
         # bug lp:376255 was reported about this.
