@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +16,13 @@
 
 
 import os
-import shutil
 
+from bzrlib import check, osutils
+from bzrlib.errors import PointlessCommit
 from bzrlib.tests import (
     SymlinkFeature,
     TestCaseWithTransport,
     )
-from bzrlib.branch import Branch
-from bzrlib.errors import PointlessCommit, BzrError
-from bzrlib.tests.test_revision import make_branches
-from bzrlib import osutils
 
 
 class TestCommitMerge(TestCaseWithTransport):
@@ -46,7 +43,7 @@ class TestCommitMerge(TestCaseWithTransport):
         wtx.commit('commit one', rev_id='x@u-0-1', allow_pointless=True)
         wty.commit('commit two', rev_id='y@u-0-1', allow_pointless=True)
 
-        self.assertEqual((1, []), by.fetch(bx))
+        by.fetch(bx)
         # just having the history there does nothing
         self.assertRaises(PointlessCommit,
                           wty.commit,
@@ -96,10 +93,8 @@ class TestCommitMerge(TestCaseWithTransport):
         self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
         self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
 
-        bx.check()
-        by.check()
-        bx.repository.check([bx.last_revision()])
-        by.repository.check([by.last_revision()])
+        check.check_dwim(bx.base, False, True, True)
+        check.check_dwim(by.base, False, True, True)
 
     def test_merge_with_symlink(self):
         self.requireFeature(SymlinkFeature)
