@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,26 +15,19 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import os
 import warnings
 
 from bzrlib import (
     bugtracker,
     revision,
-    symbol_versioning,
     )
-from bzrlib.branch import Branch
 from bzrlib.errors import (
     InvalidBugStatus,
     InvalidLineInBugsProperty,
-    NoSuchRevision,
     )
-from bzrlib.deprecated_graph import Graph
-from bzrlib.revision import (find_present_ancestors,
-                             NULL_REVISION)
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.workingtree import WorkingTree
 
 # We're allowed to test deprecated interfaces
 warnings.filterwarnings('ignore',
@@ -208,31 +201,8 @@ class TestRevisionMethods(TestCase):
         self.assertEqual('a', r.get_summary())
         r.message = '\na\nb'
         self.assertEqual('a', r.get_summary())
-
-    def test_get_apparent_author(self):
-        r = revision.Revision('1')
-        r.committer = 'A'
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual('A', author)
-        r.properties['author'] = 'B'
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual('B', author)
-        r.properties['authors'] = 'C\nD'
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual('C', author)
-
-    def test_get_apparent_author_none(self):
-        r = revision.Revision('1')
-        author = self.applyDeprecated(
-                symbol_versioning.deprecated_in((1, 13, 0)),
-                r.get_apparent_author)
-        self.assertEqual(None, author)
+        r.message = None
+        self.assertEqual('', r.get_summary())
 
     def test_get_apparent_authors(self):
         r = revision.Revision('1')
