@@ -82,7 +82,7 @@ except ImportError, e:
 else:
     from paramiko.sftp import (SFTP_FLAG_WRITE, SFTP_FLAG_CREATE,
                                SFTP_FLAG_EXCL, SFTP_FLAG_TRUNC,
-                               CMD_HANDLE, CMD_OPEN)
+                               SFTP_OK, CMD_HANDLE, CMD_OPEN)
     from paramiko.sftp_attr import SFTPAttributes
     from paramiko.sftp_file import SFTPFile
 
@@ -389,6 +389,11 @@ class SFTPTransport(ConnectedTransport):
                                          self._host, self._port)
         return connection, (user, password)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get_sftp(self):
         """Ensures that a connection is established"""
         connection = self._get_connection()
@@ -715,6 +720,8 @@ class SFTPTransport(ConnectedTransport):
             if (e.args[0].startswith('Directory not empty: ')
                 or getattr(e, 'errno', None) == errno.ENOTEMPTY):
                 raise errors.DirectoryNotEmpty(path, str(e))
+            if e.args == ('Operation unsupported',):
+                raise errors.TransportNotPossible()
             mutter('Raising exception with args %s', e.args)
         if getattr(e, 'errno', None) is not None:
             mutter('Raising exception with errno %s', e.errno)
@@ -810,10 +817,32 @@ class SFTPTransport(ConnectedTransport):
         """Return the stat information for a file."""
         path = self._remote_path(relpath)
         try:
-            return self._get_sftp().stat(path)
+            return self._get_sftp().lstat(path)
         except (IOError, paramiko.SSHException), e:
             self._translate_io_exception(e, path, ': unable to stat')
 
+    def readlink(self, relpath):
+        """See Transport.readlink."""
+        path = self._remote_path(relpath)
+        try:
+            return self._get_sftp().readlink(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': unable to readlink')
+
+    def symlink(self, source, link_name):
+        """See Transport.symlink."""
+        try:
+            conn = self._get_sftp()
+            sftp_retval = conn.symlink(source, link_name)
+            if SFTP_OK != sftp_retval:
+                raise TransportError(
+                    '%r: unable to create symlink to %r' % (link_name, source),
+                    sftp_retval
+                )
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, link_name,
+                                         ': unable to create symlink to %r' % (source))
+
     def lock_read(self, relpath):
         """
         Lock the given file for shared (read) access.
